\name{PVM.barrier}
\alias{.PVM.barrier}
\title{Group synchronization}
\description{
 Blocks the calling process until all processes in a group have called
 it.
}
\usage{
.PVM.barrier (group, count = .PVM.gsize (group))
}
\arguments{
  \item{group}{a character string naming the group}
  \item{count}{the number of group members that must call
    \code{.PVM.barrier} before they are all released, usually the total
    number of members of the specified group.}
}
\value{
  None.
}
\details{
  \code{.PVM.barrier} blocks the calling process until count members of
  the group have called it.  The count argument is required because
  processes could be joining the given group after other processes have
  called \code{.PVM.barrier}.  Thus PVM doesn't know how many group
  members to wait for at any given instant. Although count can be set
  less, it is typically the total number of members of the group.  So
  the logical function of the \code{.PVM.barrier} call is to provide a
  group synchronization.  During any given barrier call all
  participating group members must call barrier with the \emph{same} count
  value.  Once a given barrier has been successfully passed,
  \code{.PVM.barrier} can be called again by the same group using the
  same group name.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}
}
\examples{
gname <- "pvmtest"
\dontrun{myinst <- .PVM.joingroup (gname)}
# do something here ...
\dontrun{.PVM.barrier (gname, 5)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.bcast}
\alias{.PVM.bcast}
\title{Broadcasting  the  data}
\description{
  Broadcasts the data in the active message buffer to a group of
  processes.
}
\usage{
.PVM.bcast (group, msgtag)
}
\arguments{
  \item{group}{a character string naming the group}
  \item{msgtag}{an integer identifying the message}
}
\value{
  None.
}
\details{
   \code{.PVM.bcast} broadcasts a message stored in the active send
   buffer to all the members of \code{group}.  The broadcast message is
   not sent back to the sender.  Any PVM task can call
   \code{.PVM.bcast}, it need not be a member of the group.  The content
   of the message can be distinguished by \code{msgtag}.  At the other
   end, \code{\link{.PVM.recv}} or other receiving functions can be used
   to receive the message, just as the ones sent by
   \code{\link{.PVM.send}}.

   \code{.PVM.bcast} is asynchronous.  Computation on the sending
   processor resumes as soon as the message is safely on its way to the
   receiving processors.  This is in contrast to synchronous
   communication, during which computation on the sending processor
   halts until a matching receive is exe cuted by all the receiving
   processors.

   \code{.PVM.bcast} first determines the tids of the group members by
   checking a group data base.  A multicast is performed to these tids.
   If the group is changed during a broadcast the change will not be
   reflected in the broadcast.  Multicasting is not supported by most
   multiprocessor vendors.  Typically their native calls only support
   broadcasting to all the user's processes on a multiprocessor. Because
   of this omission, \code{.PVM.bcast} may not be an efficient
   communication method on some multiprocessors.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}
}
\examples{
gname <- "pvmtest"
\dontrun{.PVM.initsend ()}
\dontrun{.PVM.pkintvec (1:10)}
\dontrun{.PVM.bcast(gname, msgtag = 11)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.buffers}
\alias{.PVM.getsbuf}
\alias{.PVM.getrbuf}
\alias{.PVM.setsbuf}
\alias{.PVM.setrbuf}
\title{Manipulating Message Buffers}
\description{
  Get or set the id of the active send/receive buffer.
}
\usage{
.PVM.getsbuf ()
.PVM.getrbuf ()
.PVM.setsbuf (bufid)
.PVM.setrbuf (bufid)
}
\arguments{
  \item{bufid}{integer message buffer id}
}
\value{
  The \emph{get} functions return the id of active send/receive buffer.
  The \emph{set} functions return the id the old send/receive buffer
  (-1 indicate an error).
}
\details{
  The function \code{.PVM.getsbuf} returns the id of the active send
  buffer or 0 if no current buffer.

  The function \code{.PVM.getrbuf} returns the id of the active receive
  buffer or 0 if no current buffer.

  The function \code{.PVM.setrbuf} switches the active receive buffer to
  \code{bufid} and saves the previous active receive buffer and returns
  its id.  If \code{bufid} is set to 0, the present active receive
  buffer is saved and no active receive buffer exists.

  The function \code{.PVM.setsbuf} switches the active send buffer to
  \code{bufid} and saves the previous active send buffer and returns its
  id.  If \code{bufid} is set to 0, the present active send buffer is
  saved and no active send buffer exists.

  The \code{set} functions are required to manage multiple message buffers. 
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.mkbuf}}, \code{\link{.PVM.freebuf}}
}
\examples{
\dontrun{sbufid <- .PVM.getsbuf ()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.bufinfo}
\alias{.PVM.bufinfo}
\title{Message buffer infomation}
\description{
  Returns information about the requested message buffer.
}
\usage{
.PVM.bufinfo (bufid)
}
\arguments{
  \item{bufid}{id of a particular message buffer}
}
\value{
  Return a list (\code{bytes}, \code{msgtag}, \code{tid}) of information
  about the requested message buffer.
  \item{bytes}{the length in bytes of the entire message}
  \item{msgtag}{the message label}
  \item{tid}{the source of the message}
}
\details{
  Returns information about the requested message buffer.  Typically it
  is used to determine about the last received message such as it size
  or source.  It is especially useful when an application is able to
  receive any incoming message (with the use of wildcards).
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.send}}, \code{\link{.PVM.nrecv}},
  \code{\link{.PVM.recv}}
}
\examples{
\dontrun{bufid <- .PVM.recv (-1, -1)}
\dontrun{info <- .PVM.bufinfo (bufid)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.config}
\alias{.PVM.config}
\title{PVMD configuration}
\description{
  Query the configuration of the parallel virtual machine.
}
\usage{
.PVM.config ()
}
\arguments{
  None.
}
\value{
  Returns the configuration information as a data frame, including each
  host's pvmd task id, name, architecture and relative speed.  One row
  per host.
}
\details{
  \code{.PVM.config} returns information about the present virtual
  machine, similar to that available from the PVM console command
  \code{conf}.
}
\references{
  PVM documentation
}
\examples{
try (.PVM.config ())
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.exit}
\alias{.PVM.exit}
\title{Unregister process from local PVM deamon.}
\description{
  Tells the local pvmd that this process is leaving PVM.
}
\usage{
.PVM.exit()
}
\arguments{
  None.
}
\value{
  Return 0 when successfully unregistered, -1 when failed.
}
\details{
  The function \code{.PVM.exit} tells local pvmd that this process is
  leaving PVM.  This process is not killed but can continue to perform
  other tasks.

  \code{.PVM.exit} should be called by all PVM process before they stop
  or exit for good.  It \emph{must} be called by processes what were not
  started with \code{.PVM.spawn}, such as the master \code{R} process.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.mytid}}
}
\examples{
## Finished with PVM
\dontrun{.PVM.exit()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.freebuf}
\alias{.PVM.freebuf}
\title{Free message buffer}
\description{
  Disposes of a message buffer.
}
\usage{
.PVM.freebuf (bufid)
}
\arguments{
  \item{bufid}{integer message buffer id}
}
\value{
  Returns 0 if successful or -1 if failed.
}
\details{
  This function frees the memory associated with the message buffer
  identified by \code{bufid}.  Message buffers are created by
  \code{\link{.PVM.mkbuf}}, \code{\link{.PVM.initsend}} and
  \code{\link{.PVM.recv}}.

  \code{\link{.PVM.freebuf}} should be called for a send buffer created
  by \code{\link{.PVM.mkbuf}} after the message has been send and is no
  longer needed.

  Receive buffer typically do not have to be freed unless they have been
  saved in the cources of using multiple buffers.  But
  \code{\link{.PVM.freebuf}} can be used to destroy receive buffers as
  well.  Therefore, messages that arrive but are no longer needed as a
  result of some other event can be destroyed to save buffer space.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.initsend}}, \code{\link{.PVM.mkbuf}},
  \code{\link{.PVM.setrbuf}},  \code{\link{.PVM.recv}} 
}
\examples{
\dontrun{bufid <- .PVM.mkbuf ("Raw")}
## Send the message ...
\dontrun{.PVM.freebuf (bufid)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.gather}
\alias{.PVM.gather}
\title{Gather the data into root}
\description{
  A specified member of the group (the root) receives mes sages from
  each member of the group and gathers these messages into a single
  array.
}
\usage{
.PVM.gather (x, count = length (x), msgtag, group, rootginst = 0)
}
\arguments{
  \item{x}{an integer or double vector of length at least \code{count}}
  \item{count}{the number of elements to be sent to the root}
  \item{msgtag}{an integer message tag supplied by the user.}
  \item{group}{a character string naming the group}
  \item{rootginst}{an integer instance number of group member who
    performs the gather of its array to the members of the group.}
}
\value{
  On the root, a vector combining all the \code{x}'s.
}
\details{
   \code{.PVM.gather} performs a send of messages from each member of
   the group to the specified root member of the group.  All group
   members must call \code{.PVM.gather}, each sends its vector data of
   to the root which accumulates these messages into a single vector.
   The root task is identified by its instance number in the group.

   \code{x} has to be a vector of storage mode integer or double.  The
   \code{.PVM.gather.default} function just calls \code{stop ()}.

   \code{.PVM.gather} does not block.  If a task calls
   \code{.PVM.gather} and then leaves the group before the root has
   called \code{.PVM.gather} an error may occur.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}, \code{\link{.PVM.reduce}},
  \code{\link{.PVM.scatter}}
}
\examples{
gname <- "pvmtest"
\dontrun{
if (myinum == 0) {
   result <- .PVM.gather(as.integer (1:100),
                         msgtag = 11, group = gname, root = 0)
}
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.getinst}
\alias{.PVM.getinst}
\title{Instance number identified by group name and task id}
\description{
  Get the instance numbers identified by a group name and task ids}
}
\usage{
.PVM.getinst (group, tids = .PVM.mytid ())
}
\arguments{
  \item{group}{a character string naming the group}
  \item{tids}{a vector of integer task ids}
}
\value{
  a vector of instance numbers
}
\details{
  \code{.PVM.gettid} returns the corresponding instance number of the
  PVM process identified by the group name \code{group} and task ids
  \code{tids}. It can be called by any task whether in the group or not.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}, \code{\link{.PVM.gettid}}
}
\examples{
gname <- "pvmtest"
\dontrun{myinum <- .PVM.getinst (gname)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.gettid}
\alias{.PVM.gettid}
\title{Task id identified by group name and instance number}
\description{
  Get the task id identified by a group name and an instance number}
}
\usage{
.PVM.gettid (group, inum = 0:(.PVM.gsize(group) - 1))
}
\arguments{
  \item{group}{a character string naming the group}
  \item{inum}{a vector of integer instance numbers}
}
\value{
  a vector of task ids.
}
\details{
  \code{.PVM.gettid} returns the tid of the PVM process identified by
  the group name \code{group} and the instance number \code{inum}.
}
\references{
  PVM documentation
}
\note{
  \code{inum} is given a default value corresponding to all instances in
  the group.  It is not valid when the instances numbers are not
  continues (when, for example, some processes already left the group).
}
\seealso{
   \code{\link{.PVM.joingroup}}, \code{\link{.PVM.getinst}}
}
\examples{
gname <- "pvmtest"
leader <- 0
\dontrun{leadtid <- .PVM.gettid (gname, leader)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.group}
\alias{.PVM.joingroup}
\alias{.PVM.lvgroup}
\title{Join or leave a names group}
\description{
  Enrolls or unenrolls the calling process to a named group.
}
\usage{
.PVM.joingroup (group)
.PVM.lvgroup (group)
}
\arguments{
  \item{group}{a character string naming the group}
}
\value{
  \code{.PVM.joingroup} returns the instance number (rank) of the the
  process in the group.

  \code{.PVM.lvgroup} returns no value.
}
\details{
  \code{.PVM.joingroup} enrolls the calling task in the group named
  \code{group} and returns the instance number (\code{inum}) this task
  in this group.

  Instance numbers start at 0 and count up.  When using groups a
  (\code{group}, \code{inum}) pair uniquely identifies a PVM process.
  If a task leaves a group by calling \code{.PVM.lvgroup} and later
  rejoins the same group, the task is not guaranteed to get the same
  instance number.  PVM attempts to reuse old instance numbers, so when
  a task joins a group it will get the lowest available instance number.
  A task can be a member of multiple groups simultaneously.

  \code{.PVM.lvgroup} unenrolls the calling task from the group named
  \code{group}.
}
\references{
  PVM documentation
}
\note{
  If the process fails to join a group with the following message,
  \code{
    libpvm [t40002]: gs_getgstid() failed to start group server: No such  file
  }
  It means pvmd cannot find the executable \code{pvmgs}, put its path to
  the \code{ep=} option of the host file.
}
\seealso{
}
\examples{
gname <- "pvmtest"
\dontrun{myinum <- .PVM.joingroup (gname)}
\dontrun{.PVM.lvgroup (gname)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.gsize}
\alias{.PVM.gsize}
\title{Get the size of the group}
\description{
  Get the number of processes currently in the group
}
\usage{
.PVM.gsize (group)
}
\arguments{
  \item{group}{a character string naming the group}
}
\value{
  Returns the number of processes currently in the group.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}, \code{\link{.PVM.lvgroup}}
}
\examples{
gname <- "pvmtest"
\dontrun{size <- .PVM.gsize(gname)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.initsend}
\alias{.PVM.initsend}
\alias{.PVM.encoding}
\title{Initialize send buffer}
\description{
  Clears default send buffer and specifies message sending.
}
\usage{
.PVM.initsend (encoding = c("Default", "Raw", "InPlace")
.PVM.encoding
}
\arguments{
  \item{encoding}{a character string specifying the next message's
  encoding scheme.  Must be one of \code{"Default"} (default),
  \code{"Raw"} or \code{"InPlace"}.}
}
\value{
  Returns message buffer id. -1 if there was an error.
}
\details{
  This function clears the send buffer and prepares it for packing a new
  message.

  Possible encoding themes are \code{"Default"} (default), \code{"Raw"}
  or \code{"InPlace"}. If the user knows that the next message will be
  sent only to a machine that understands the native format, he can use
  \code{"Raw"} to save some encoding costs.

  \code{"InPlace"} encoding specifies that data be left in place during
  packing.  The message buffer contains only the size and pointers to
  the items be send.  When \code{\link{.PVM.send}} is called, the items
  are copied directly out of the user's memory.  This option decreases
  the number of times a message is copied, at the expense of requiring
  that the user not modify the items between the time they are packed
  and the time they are sent.

  \code{.PVM.encoding} is a mapping of the strings to integers (constants
  defined by PVM).
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.mkbuf}}
}
\examples{
a <- 1:10
\dontrun{bufid <- .PVM.initsend ("InPlace")}
\dontrun{.PVM.pkintvec (a)}
\dontrun{.PVM.send (tid, msgtag)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.kill}
\alias{.PVM.kill}
\title{Kill pvm process}
\description{
  Terminate a specified PVM process.
}
\usage{
.PVM.kill(tids)
}
\arguments{
  \item{tids}{task ids of the PVM processes to be killded.}
}
\value{
  Return number of processes successfully killed.
}
\details{
  \code{.PVM.kill} sends a terminate (\code{SIGTERM}) signal to the PVM
  process identified by \code{tid}.  It is not designed to kill the
  calling process.  To kill your self, call \code{\link{.PVM.exit}()}
  followed by \code{q()}.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.exit}}
}
\examples{
## kill one of spawned children
\dontrun{.PVM.kill (children[1])}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.mcast}
\alias{.PVM.mcast}
\title{Multicast data}
\description{
  Multicast the data in the active message buffer to a set of tasks.
}
\usage{
.PVM.mcast (tids, msgtag)
}
\arguments{
  \item{tids}{vector of the task ids of the tasks to be sent to}
  \item{msgtag}{integer message tag supplied by the user, must be >=0}
}
\value{
  Return 0 if successful, -1 if failed.
}
\details{
  \code{.PVM.mcast} multicasts a message stored in the active send
  buffer to tasks specified in the vector \code{tids}.  The message is
  not send to the caller even if its tid is in \code{tids}.  The content
  of the message can be dustinguished by \code{msgtag}.

  The receiving processes can call either \code{\link{.PVM.recv}} or
  \code{\link{.PVM.nrecv}} to receive their copy of the multicast.
  \code{.PVM.mcast} is asynchronous.

  Multicasting is not supported by most multiprocessor vendors.
  Typically their native calls support only broadcasting to \emph{all}
  the user's processes on a multiprocessor.  Because of this omission,
  \code{.PVM.mcast} may not be an efficient communication method on some
  multiprocessors except in the special case of broadcasting to all PVM
  processses.    
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.send}}
}
\examples{
\dontrun{.PVM.initsend ("Raw")}
\dontrun{.PVM.pkintvec (1:10)}
\dontrun{.PVM.mcast (tids, msgtag = 5)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.mkbuf}
\alias{.PVM.mkbuf}
\title{Create message buffer}
\description{
  Creates a new message buffer
}
\usage{
.PVM.mkbuf (encode = c("Default", "Raw", "InPlace"))
}
\arguments{
  \item{encode}{a character string specifying the next message's
    encoding scheme.  Must be one of \code{"Default"} (default),
    \code{"Raw"} or \code{"InPlace"}.}
}
\value{
  Returns message buffer id or -1 if failed..
}
\details{
  This function creates a new message buffer and sets its encoding theme
  (see \code{\link{.PVM.initsend}}.

  It is useful if the user wishes to manage multiple message buffers and
  should be used in conjunction with \code{\link{.PVM.freebuf}}.
  \code{\link{.PVM.freebuf}} should be called for a send buffer after a
  message has been send and is no longer needed.

  Receive buffer are created automatically by \code{\link{.PVM.recv}}
  and \code{\link{.PVM.nrecv}} routines and do not have to be freed
  unless they have been explicitly saved with \code{\link{.PVM.setrbuf}}.

  Typically multiple send and receive buffer are not needed and the user
  can simply use the \code{\link{.PVM.initsend}} to reset the default
  send buffer.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.initsend}},\code{\link{.PVM.freebuf}},
  \code{\link{.PVM.setrbuf}}
}
\examples{
\dontrun{bufid <- .PVM.mkbuf ("Raw")}
## Send the message ...
\dontrun{.PVM.freebuf (bufid)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.mstats}
\alias{.PVM.mstats}
\title{Status of host machines}
\description{
   Check the status of hosts computers in the virtual machine.
}
\usage{
.PVM.mstats (hosts)
}
\arguments{
  \item{hosts}{vector of host names (strings)}
}
\value{
  Return machine status. \code{"OK"} if host is running, \code{"Not in
  VM"} if host is not in virtual machine, \code{"Not reachable"} is host
  is unreachable (and possibly failed).
}
\details{
  \code{.PVM.mstats} returns the status the computers named by
  \code{hosts} with respect to running PVM processes.  This routine can
  be used to determine if a particular host has failed and if the
  virtual machine needs to be reconfigured. \code{\link{.PVM.notify}}
  can also be used to notify the caller that a host has failed.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.notify}}
}
\examples{
\dontrun{.PVM.mstats ("abacus")}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.mytid}
\alias{.PVM.mytid}
\title{Current task id}
\description{
  Returns the \code{tid} of the current process.
}
\usage{
.PVM.mytid()
}
\arguments{
  None.
}
\value{
  Returns integer task id of the calling process.  Return -1 if an error
  occured.
}
\details{
  \code{.PVM.mytid} returns the \code{tid} of the calling process and
  can be called multiple times.  The \code{tid} is a positive integer
  created by local pvmd.
  
  This function enrolls this process into PVM on its first call and
  generate a unique \code{tid} if this process was not created by
  \code{\link{.PVM.spawn}}.  In fact, any PVM system call will enroll a
  task in PVM if the task is not enrolled before the call.
}

\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.exit}}, \code{\link{.PVM.spawn}}
}
\examples{
## For name(tid) of process and to register with PVM
\dontrun{mytid <- .PVM.mytid()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.notify}
\alias{.PVM.notify}
\title{Monitor pvmd}
\description{
  Request notification of PVM event such as host failure.
}
\usage{
.PVM.notify (msgtag, what = c("ExitTask", "DeleteHost", "AddHost"),
             par = 0)
}
\arguments{
  \item{msgtag}{message tag to be used in notification}
  \item{what}{a character string specifying the type of event to trigger
    the notification, must be one of \code{"ExitTask"} (default),
    \code{"DeleteHost"}, or \code{"AddHost"}.}
  \item{par}{additional parameter, if \code{what} = \code{"AddHost"},
    specify the number of times to notify, if \code{what} =
    \code{"DeleteHost"} or \code{"ExitTask"}, specify a vector of host
    id or task id.}
}
\value{
  Status code returned by the routine.  -1 if there was an error.
}
\details{
  The routine \code{.PVM.notify} requests PVM to notify the caller on
  detecting certain events.  One or more notify messages (see below) are
  sent by PVM back to the calling task.  The messages have tag
  \code{msgtag} supplied to notify.
  
  The notification messages have the following format:
  \itemize{
    \item{\code{"ExitTask"}}{One notify message for each task id requested.
      The message body contains a single integer task id of exited task.}
    \item{\code{"DeleteHost"}}{One notify message for each host id
      requested. The message  body  contains a single integer host
      (pvmd) id of exited pvmd.}
    \item{\code{"AddHost"}}{\code{par} notify messages are sent, one
      each time the local pvmd's host table is updated. The message is a
      vector of ids of new pvmds and can be unpacked by
      \code{\link{.PVM.upkintvec}}.  The  counter of \code{"AddHost"}
      messages yet to be sent is  replaced  by successive calls to
      \code{.PVM.notify}.  Specifying \code{par} = -1 turns on
      \code{"AddHost"} messages until a future notify; 0 disables them.}
    
    The calling task is responsible for receiving messages with the
    specified tag and taking appropriate action.  This feature makes PVM
    fault tolerant.
  }
}
\references{
  PVM documentation
}
\seealso{
  \link{PVMD}
}
\examples{
\dontrun{.PVM.notify (msgtag = 999, what = "Exit", tids)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.nrecv}
\alias{.PVM.nrecv}
\title{Nonblocking receive}
\description{
  Checks for nonblocking message.
}
\usage{
.PVM.nrecv (tid = -1, msgtag = -1)
}
\arguments{
  \item{tid}{tid of sending process (-1 matches any tid)}
  \item{msgtag}{integer message tag (>=0) supplied by the user (-1
  matches any message tag)}
}
\value{
  Returns the id of the new active receive buffer. -1 indicates an error.
}
\details{
  \code{.PVM.nrecv} checks to see whether a message with lable
  \code{msgtag} has arrived from \code{tid}.  If a matching message has
  arrived, it then immediately places the message in a new \emph{active}
  receive buffer, which also clears the current receive buffer, if any,
  and returns the buffer id.

  If the requested message has not arrived, then \code{.PVM.nrecv}
  immediately returns 0.
  
  A -1 in \code{tid} or \code{msgtag} matches anything.  Later
  \code{\link{.PVM.bufinfo}} can be used to find out the tid of the
  sending process or msgtag.

  \code{.PVM.nrecv} is nonblocking in the sense that the routing always
  returns immediately either with the message or with the information
  that the message has not arrived at the local pvmd yet.
  \code{.PVM.nrecv} can be called multiple times to check whether a
  given message has arrived yet.

  If \code{.PVM.nrecv} returns with the message, the data in the message
  can be unpacked with the \code{\link{PVM.unpack}} functions.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.send}}, \code{\link{.PVM.recv}},
  \code{\link{.PVM.bufinfo}}, \code{\link{PVM.unpack}}
}
\examples{
\dontrun{myparent <- .PVM.parent ()}
\dontrun{while (1) {
    bufid <- .PVM.nrecv (myparent, 4)
    if (bufid > 0) {
        data <- .PVM.upkintvec ()
        # do something ...
        break
    } else {
        # do something else
    }
}}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.options}
\alias{PVM.options}
\title{libpvm Options}
\description{
  Get or set the value of libpvm options.
}
\usage{
PVM.options (what, val)
}
\arguments{
  \item{what}{a character string specifying libpvm options, must be one
    of the following,
    \itemize{
      \item{\code{"Route"}}{Message routing policy}
      \item{\code{"DebugMask"}}{Libpvm debug mask}
      \item{\code{"AutoErr"}}{Auto error reporting}
      \item{\code{"OutputTid"}}{Stdout destination for children}
      \item{\code{"OutputCode"}}{Output message tag}
      \item{\code{"TraceTid"}}{Trace data destination for children}
      \item{\code{"TraceCode"}}{Trace message tag}
      \item{\code{"FragSize"}}{Message fragment size}
      \item{\code{"ResvTids"}}{Allow messages to reserved tags and TIDs}
      \item{\code{"SelfOutputTid"}}{Stdout destination}
      \item{\code{"SelfOutputCode"}}{Output message tag}
      \item{\code{"PvmSelfTraceTid"}}{Trace data destination}
      \item{\code{"SelfTraceCode"}}{Trace message tag}
      \item{\code{"ShowTids"}}{\code{pvm_catchout} prints task ids with output}
      \item{\code{"PollType"}}{Message wait policy (shared memory)}
      \item{\code{"PollTime"}}{Message spinwait duration}
    }
  }
  \item{val}{integer value of the option}
}
\value{
  Returns the current option value.  If \code{val} is present, set the
  corresonding option to new value \code{val}.
}
\details{
  \code{.PVM.options} sets or queries miscellaneous options in the PVM
  library.  The options are,
  \enumerate{    
    \item{\code{"Route"}}{
      Possible values are:
      \itemize{
        \item{1}{"RouteDirect", set up direct task-to-task links
          (using TCP) for all subsequent  communication.}
        \item{2}{"DontRouteDirect", communication through PVM daemon.}
        \item{3}{"AllowDirect" (the default), this setting on task A
        allows other tasks to set up direct links to A.}
      }
    }
    \item{\code{"DebugMask"}}{
      When debugging is turned on, PVM will log detailed information
      about its operations and progress on its stderr stream.  The value
      is the debugging level.  Default is 0, not to print any debug
      information.
    }
    \item{\code{"AutoErr"}}{
      When an error results from a libpvm function call and
      \code{"AutoErr"} is set to 1 (the default), an error message is
      automatically printed on stderr.  A set ting of 0 disables this.
      A setting of 2 causes the library to terminate the task by calling
      exit() after printing the error message.  A setting of 3 causes
      the library to abort after printing the error message.
    }
    \item{\code{"OutputTid"}}{
      The stdout destination for children tasks (spawned after the
      the option is set).  Everything printed on the standard output of
      tasks spawned by the calling task is packed into messages and sent
      to the destination.  \code{val} is the TID of a PVM task.  Setting
      PvmOutputTid to 0 redirects stdout to the master pvmd, which
      writes to the log file /tmp/pvml.<uid> The default setting is
      inherited from the parent task, else is 0.
    }
    \item{\code{"OutputCode"}}{
       The message tag for standard output messages.  Should only
       be set when a task has \code{"OutputTid"} set to itself.
    }
    \item{\code{"TraceTid"}}{
      The trace data message destination for chilren tasks (spawned
      after the option is set).  Libpvm trace data is sent as messages
      to the destination.  \code{val} is the TID of a PVM task.  Setting
      \code{"TraceTid"} to 0 discards trace data.  The default setting
      is inherited from the parent task, else is 0.
    }
    \item{\code{"TraceCode"}}{     
      The message tag for trace data messages.  Should only be set when
      a task has \code{"TraceTid"} set to itself.
    }
    \item{\code{"FragSize"}}{    
      Specifies the message fragment size in bytes.  Default value
      varies with host architecture.
    }
    \item{\code{"ResvTids"}}{
      A value of 1 enables the task to send messages with reserved tags
      and to non-task destinations.  The default (0) causes libpvm to
      generate a \code{"BadParam"} error when a reserved identifier is
      specified.
    }
    \item{\code{"SelfOutputTid"}}{
      Sets the \code{stdout} destination for the task.  Every thing
      printed on \code{stdout} is packed into messages and sent to the
      destination.  Note: this only works for spawned tasks, because the
      pvmd doesn't get the output from tasks started by other means.
      The value is the TID of a PVM task.  Setting
      \code{"SelfOutputTid"} to 0 redirects stdout to the master pvmd,
      which writes to the log file /tmp/pvml.<uid>.  The default setting
      is inherited from the parent task, else is 0.  Setting either
      \code{"SelfOutputTid"} or \code{"SelfOutputCode"} also causes both
      \code{"OutputTid"} and \code{"OutputCode"} to take on the values
      of \code{"SelfOutputTid"} and \code{"SelfOutputCode"},
      respectively.
    }
    \item{\code{"SelfOutputCode"}}{
      The message tag for standard output messages.
    }
    \item{\code{"PvmSelfTraceTid"}}{
     The trace data message destination for the task.  Libpvm trace data
     is sent as messages to the destination.  The value is the TID of a
     PVM task.  Set ting \code{"SelfTraceTid"} to 0 discards trace data.
     The default setting is inherited from the parent task, else is 0.
     Setting either \code{"SelfTraceTid"} or \code{"SelfTraceCode"} also
     causes both \code{"TraceTid"} and \code{"TraceCode"} to take on the
     values of \code{"SelfTraceTid"} and \code{"SelfTraceCode"},
     respectively.
    }
    \item{\code{"SelfTraceCode"}}{
      The message tag for trace data messages.
    }
    \item{\code{"ShowTids"}}{
      If true (nonzero), \code{pvm_catchout} (note: not supported by
      rpvm) tags each line of output printed by a child task with the
      task id.  Otherwise, output is exactly as printed.
    }
    \item{\code{"PollType"}}{
      The message wait policy when using shared-memory message
      transport.  Setting \code{"PollType"} to \code{"PollConstant"}
      causes the application to spin on its message queue waiting for a
      message.  Setting \code{"PollType"} to \code{"PollSleep"} causes
      the application to poll the message queue for messages
      \code{"PollTime"} times before pending on the semaphore.
    }
    \item{\code{"PollTime"}}{
      The poll count for applications checking their message queue
      before they pend on the semaphore.  This option is ignored if
      \code{"PollType"} is set to \code{"PollConstant"}.
    }
  }
} 
\references{
  PVM documentation
}
\examples{
\dontrun{PVM.options (what = "DebugMask")}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.pack}
\alias{PVM.pack}
\alias{.PVM.pkdouble}
\alias{.PVM.pkint}
\alias{.PVM.pkstr}
\alias{.PVM.pkintvec}
\alias{.PVM.pkdblvec}
\alias{.PVM.pkstrvec}
\alias{.PVM.pkintmat}
\alias{.PVM.pkdblmat}
\alias{.PVM.pkstrmat}
\alias{.PVM.pkfactor}
\title{Packing data}
\description{
  Pack data into current active send buffer.
}
\usage{
.PVM.pkdouble (data = 0.0, stride = 1)
.PVM.pkint (data = 0, stride = 1)
.PVM.pkstr (data = "")

.PVM.pkintvec (data)
.PVM.pkdblvec (data)
.PVM.pkstrvec (data)

.PVM.pkintmat (data)
.PVM.pkdblmat (data)
.PVM.pkstrmat (data)

.PVM.pkfactor (data)
}
\arguments{
  \item{data}{data to be packed.}
  \item{stride}{the stride to be used when packing the items.  For
    example, if \code{stride = 2} with \code{.PVM.pkdouble} then every
    other element in the vector \code{data} will be packed.}
}
\value{
  \code{.PVM.pkstrvec} returns the maximum length of the strings
  packed.  Others return 0 if succeeded or -1 if failed.
}
\details{
  The first three functions are low-level correspondents of the PVM
  packing routines for packing double, integer arrays and a single
  character string.  In particular, the number of item packed
  is not passed and has to be specified when unpacking the data.

  The other functions also pack the dimension information therefore
  there is no need to specify length and the correct dimension will be
  automatically recovered when corresponding unpack functions are used.
}
\note{
  The PVM C library supports a variety of data types.  Since R doesn't
  have that many types, most are irrelevant.

  To passing data between R process, there is in general no need to
  distinguish between integer and double (except for efficiency
  consideration). \code{.PVM.dblvec} and \code{.PVM.dblmat} are
  recommended for all numerical data.

  \code{.PVM.pkstr} and \code{\link{.PVM.upkstr}} are mainly for
  communicating with C functions.
  
  Special values such as NAs, NaNs, Infs, etc., are not supported at
  this point.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{PVM.unpack}}
}
\examples{
## Pack and send a matrix
a <- matrix (1:20, nrow=4)
\dontrun{.PVM.pkdblmat (a)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.parent}
\alias{.PVM.parent}
\title{Parent task id}
\description{
  Returns the \code{tid} of the parent process.
}
\usage{
.PVM.parent ()
}
\arguments{
  None.
}
\value{
  Returns tid of the process that spawned the calling process.  
}
\details{
  If the calling process was not created with \code{\link{.PVM.spawn}},
  return 0 (no parent).
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.spawn}}, \code{\link{.PVM.mytid}}
}
\examples{
\dontrun{myparent <- .PVM.parent ()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.probe}
\alias{.PVM.probe}
\title{Probe receive}
\description{
  Checks whether message has arrived.
}
\usage{
.PVM.probe(tid = -1, msgtag = -1)
}
\arguments{
  \item{tid}{the specific tid, -1 is every tid}
  \item{msgtag}{tag to use to specific, -1 is all}
}
\value{
  Returns the value of the new active receive buffer id. 0 if the
  message has not arrived.  -1 if there was an error.
}
\details{
  Checks to see if a message with label \code{msgtag} has arrived from
  \code{tid}.  If a matching message has arrived, returns a buffer id
  which can be used in a \code{\link{.PVM.bufinfo}} call to determine
  information about the message such as its source and length.

  A -1 in \code{msgtag} or \code{tid} matches anything (wildcard).

  After the message has arrived, \code{\link{.PVM.recv}} must be called
  before the messag can be unpacked into the user's memory using the
  unpack routines.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.recv}}, \code{\link{.PVM.bufinfo}}
}
\examples{
## To check a node (specified by tid) for a message
\dontrun{MsgReady <- .PVM.probe (tid, msgtag)}
## To see if any node is sending message with tag thisTag
\dontrun{AnyMessageWithThisTag <- .PVM.probe (-1, thisTag)}
## To see if node tid is sending any message.
\dontrun{AnyMessageFromThisTID <- .PVM.probe (tid, -1)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.pstats}
\alias{.PVM.pstats}
\title{Status of PVM processes}
\description{
  Returns the status of the specified PVM process.
}
\usage{
.PVM.pstats (tids)
}
\arguments{
  \item{tids}{vector of integer task id of destination process}
}
\value{
  Returns the status of the PVM processes (\code{"OK"}, \code{"Not
  Running"} or \code{"Invalid tid"}) identified by tids.
}
\details{
  The routine \code{.PVM.pstat} returns the status of the process
  identified by \code{tid}.  Also note that \code{\link{.PVM.notify}}
  can be used to notify the caller that a task has failed.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.notify}}
}
\examples{
\dontrun{tid <- .PVM.parent ()}
\dontrun{status <- .PVM.pstats (tid)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.rapply}
\alias{PVM.rapply}
\title{Parallel apply}
\description{
  Apply a function to the rows of a matrix in parallel, via PVM.
}
\usage{
PVM.rapply (X, FUN = mean, NTASK = 1)
}
\arguments{
  \item{X}{a numeric matrix}
  \item{FUN}{function to be applied on each row}
  \item{NTASK}{number of tasks to use}
}
\value{
  A vector with the same values as \code{apply (X, 1, FUN)}.
}
\details{
  This function requires the function \code{FUN} can be correctly
  evaluated in each spawned R process. 
}
\examples{
a <- matrix(rnorm(100), nrow = 10)
\dontrun{PVM.rapply (a, sum, 3)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.recv}
\alias{.PVM.recv}
\title{Blocking receive}
\description{
  Receive a message.
}
\usage{
.PVM.recv (tid = -1, msgtag = -1)
}
\arguments{
  \item{tid}{tid of sending process (-1 matches any tid)}
  \item{msgtag}{integer message tag (>=0) supplied by the user (-1
  matches any message tag)}
}
\value{
  Returns the id of the new active receive buffer. -1 indicates an error.
}
\details{
  \code{.PVM.recv} blocks the process until a message with lable
  \code{msgtag} has arrived from \code{tid}.  It then places the message
  in a new \emph{active} receive buffer, which also clears the current
  receive buffer.

  A -1 in \code{tid} or \code{msgtag} matches anything.  Later
  \code{\link{.PVM.bufinfo}} can be used to find out the tid of the
  sending process or msgtag.

  The PVM model guarantees the following about message order. If task 1
  sends message A to task 2, then task 1 sends message B to task 2,
  message A will arrive at task 2 before message B.  Moreover, if both
  tasks arrive before task 2 does a receive, then a wildcard receive
  will always return message A.

  \code{.PVM.recv} is blocking, which means the routing waits until a
  message matching the user specified tid and msgtag value arrives at
  the local pvmd.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.send}}, \code{\link{.PVM.nrecv}},
  \code{\link{.PVM.bufinfo}}
}
\examples{
\dontrun{myparent <- .PVM.parent ()}
\dontrun{bufid <- .PVM.recv (myparent, 4)}
\dontrun{data <- .PVM.upkintvec ()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.reduce}
\alias{.PVM.reduce}
\title{Reduction}
\description{
  Performs a reduction operation over members of the specified group.  
}
\usage{
.PVM.reduce (x, func = "Min", count = length (x), msgtag, group,
             rootginst = 0)
}
\arguments{
  \item{x}{an integer or double vector of length at least \code{count}}
  \item{func}{a character string specifying the name of the operation
    (function), must be one of \code{"Min"}, \code{"Max"}, \code{"Sum"},
    \code{"Product"}}
  \item{count}{the number of elements in \code{x}, must be the same
    among all members of the group}
  \item{msgtag}{an integer message tag supplied by the user.}
  \item{group}{a character string naming the group}
  \item{rootginst}{an integer instance number of group member who
    performs the reduce of its array to the members of the group.}
}
\value{
  On the root, a vector of length \code{count}.
}
\details{
  \code{.PVM.reduce} performs global operations such as max, min, sum,
  or a user provided operation on the data provided by the members of a
  group.  All group members call \code{.PVM.reduce} with the same size
  local data array which may contain one or more entries.  The root task
  is identified by its instance number in the

  When the \code{.PVM.reduce} completes, it returns a vector on the root
  which will be equal to the specified operation applied element-wise to
  the data vectors of all the group members.

  A broadcast by the root can be used if the other members of the group
  need the resultant value(s).

  \code{func} corresponds the four predefined functions in pvm,
  \code{PvmMin}, \code{PvmMax}, \code{PvmSum}, \code{PvmProduct}.  It is
  possible to definer user function in C with the following prototype,
  \code{void func(int *datatype, void *x, void *y, int *num, int
    *info)}.  However, it is not clear how to pass its name from C or how
  to use user function written in R which limits the usefulness of this
  function.
}
\note{
  \code{.PVM.reduce} does not block, a call to
  \code{\link{.PVM.barrier}} may be necessary.  For example, an error
  may occur if a task calls \code{.PVM.reduce} and then leaves the group
  before the root has completed its call to \code{.PVM.reduce}.
  Similarly, an error may occur if a task joins the group after the root
  has issued its call to \code{.PVM.reduce}.  Synchronization of the
  tasks (such as a call to \code{.PVM.barrier}) was not included within
  the \code{.PVM.reduce} implementation since this overhead is
  unnecessary in many user codes (which may already synchronize the
  tasks for other purposes).
}
\references{
  PVM documentation, http://www.netlib.org/pvm3/book/pvm-book.html
}
\seealso{
  \code{\link{.PVM.joingroup}}, \code{\link{.PVM.gather}},
  \code{\link{.PVM.scatter}}
}
\examples{
gname <- "pvmtest"
\dontrun{
if (myinum == 0) {
   result <- .PVM.reduce (as.double (rnorm (10)),
                          "Sum", msgtag = 11, group = gname, root = 0)
}
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.scatter}
\alias{.PVM.scatter}
\title{Scatter a vector across the group}
\description{
  Sends to each member of a group a section of a vector from a specified
  member of the group (the root).
}
\usage{
.PVM.scatter (x, count, msgtag, group, rootginst = 0)
}
\arguments{
  \item{x}{an integer or double vector on the root which are to be
    distributed to the members of the group.  If \code{n} is the number
    of members in the group, then this vector should be of length at
    least \eqn{n \times \mathrm{count}}{n * count}.  This argument is
    meaningful only on the root.}
  \item{count}{an integer specifying the number of elements to be sent
    to each member of the group from the root.}
  \item{msgtag}{an integer message tag supplied by the user.}
  \item{group}{a character string naming the group}
  \item{rootginst}{an integer instance number of group member who
    performs the scatter of its array to the members of the group.}
}
\value{
  a vector of length \code{count}
}
\details{
  \code{.PVM.scatter} performs a scatter of data from the specified
  root member of the group to each of the members of the group,
  including itself.  All group members must call \code{.PVM.scatter},
  each receives a portion of the data array from the root in their
  local result array.  It is as if the root node sends to the
  \emph{i}th member of the group count elements from its array data
  starting at offset \eqn{i \times \mathrm{count}}{i * count} from the
  beginning of the data array.  And, it is as if, each member of the
  group performs a corresponding receive of count values of datatype
  into its result array.  The root task is identified by its instance
  number in the group.
  
  \code{x} has to be a vector of storage mode integer or double.  The
  \code{.PVM.scatter} function just calls \code{stop ()}.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.joingroup}}, \code{\link{.PVM.gather}},
  \code{\link{.PVM.reduce}}, \code{\link{.PVM.bcast}}
}
\examples{
gname <- "pvmtest"
\dontrun{
myrow <- .PVM.scatter(as.integer (1:100), 10, msgtag = 11, group =
                      gname, root = 0)
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.send}
\alias{.PVM.send}
\title{Send data}
\description{
  Sends the data in the active message buffer.
}
\usage{
.PVM.send (tid, msgtag)
}
\arguments{
  \item{tid}{integer task id of destination process}
  \item{msgtag}{integer message tag (>=0) supplied by the user}
}
\value{
  Returns 0 if sucessful, -1 if failed..
}
\details{
  Sends a message stored in the active buffer to the PVM process
  identified by \code{tid}.  \code{msgtag} is used to label the content
  of the message.

  \code{.PVM.send} is asynchronous.  Computation on the sending processor
  resumes as soon as the message is safely on its way to the receiving
  processor.  This is in constrast to synchronous communication in which
  computation on the sending processor halts until the matching receive
  is executed by the receiving processor.

  \code{.PVM.send} first checks to see whether the destination is on the
  same machine.  If so and this host is a multiprocessor, then the
  vender's underlying message-passing routines are used to move the data
  between processes.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.mcast}}, \code{\link{.PVM.recv}}
}
\examples{
a <- 1:10
\dontrun{
bufid <- .PVM.initsend ()
.PVM.pkintvec (a)
.PVM.send (tid, msgtag)
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.serialize}
\alias{.PVM.serialize}
\alias{.PVM.unserialize}
\title{Serialize R Objects}
\description{
  Serialize to/from the current active send/receive buffer.
}
\usage{
.PVM.serialize (object, refhook = NULL)
.PVM.unserialize(refhook = NULL)
}
\arguments{
  \item{object}{object to serialize.}
  \item{refhook}{hook function for handling reference objects}
}
\value{
  \code{.PVM.serialize} returns \code{NULL}; \code{.PVM.unserialize}
  returns the unserialized object.
}
\details{
  The function \code{.PVM.serialize} writes \code{object} to the
  current active send buffer.  Sharing of reference objects is
  preserved within the object but not across separate calls to
  \code{.PVM.serialize}.  \code{unserialize} reads an object from
  \code{connection}.  \code{connection} may also be a string.

  The \code{refhook} functions can be used to customize handling of
  non-system reference objects (all external pointers and weak
  references, and all environments other than name space and package
  environments and \code{.GlobalEnv}).  The hook function for
  \code{.PVM.serialize} should return a character vector for references it
  wants to handle; otherwise it should return NULL.  The hook for
  \code{.PVM.unserialize} will be called with character vectors supplied to
  \code{.PVM.serialize} and should return an appropriate object.
}
\examples{
## Pack and send a matrix
\dontrun{
BUFTAG<-22
tid <- .PVM.mytid()
a <- matrix (1:20, nrow=4)
.PVM.initsend()
.PVM.serialize(a)
.PVM.send(tid, BUFTAG)
## Receive the matrix
.PVM.recv(tid, BUFTAG)
.PVM.unserialize()
}
}
\author{
  Luke Tierney \email{luke@stat.umn.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.spawn}
\alias{.PVM.spawn}
\alias{.PVM.spawnR}
\alias{.PVM.spawnflags}
\title{Spawn child tasks}
\description{
  Starts up \code{ntask} copies of an executable file \code{task} or
  slave R processes on the virtual machine.
}
\usage{
.PVM.spawn (task, ntask = 1, flag  = "Default", where = "",
            arglist = NULL, verbose = FALSE)
.PVM.spawnR (slave, ntask = 1, flag  = "Default", where = "",
             slavedir = "demo", outdir = "/tmp", verbose = FALSE)
.PVM.spawnflags
}
\arguments{
  \item{task}{name of an executable file}
  \item{ntask}{number of tasks to spawn}
  \item{flag}{options flag}
  \item{where}{name of a host or architecture}
  \item{arglist}{a character vectpr to be passed to the spawned task.}
  \item{slave}{name of slave \code{R} script}
  \item{slavedir}{full path to the slave script}
  \item{outdir}{full path to the directory where the output from slave R
    process will be}
  \item{verbose}{if true, print out more information which might be
    helpful in debugging}
}
\value{
  A vector containing \code{tids} of successfully spawned processes (the
  length could be smaller than the number requested) or -1 if there is
  an system error.
}
\details{
  \emph{flag} is a list of strings used to specify options.  Allowed values 
  are
  \tabular{ll}{
    "Default" \tab PVM chooses where to spawn processes \cr
    "Host"    \tab spawn on a particular host (\code{where}) \cr
    "Arch"    \tab spawn on a particular \code{PVM_ARCH} (\code{where}) \cr
    "Debug"   \tab starts tasks under a debugger \cr
    "Trace"   \tab trace data is generated \cr
    "MppFront"    \tab starts tasks on MPP front-end \cr
    "HostCompl"   \tab complements host set in \emph{where} 
  }

  \code{task} is a character string containing the name of the
  executable file name of the PVM process to be started.  It must
  already reside on the host on which it is to be started and can be
  find by local pvmd (possibly by specifying \code{ep=} option when
  the host is added, see \code{\link{.PVM.addhosts}}).  The default
  location PVM looks at is \code{$PVM_ROOT/bin/$PVM_ARCH}.

  To spawn a R process, one can use the script \file{slaveR.sh} which
  has to be included in the search path of pvmd, for example by passing
  \code{ep="$R_LIBS/rpvm"} (the default place this script is installed)
  to \code{.PVM.addhosts}.  Note that here \code{"R_LIBS"} referes to
  the R library on the \emph{slave} node.

  \code{.PVM.spawnR} can be used to conveniently spawn slave \code{R}
  processes.  The paths for the slave script and output file should be
  given as arguments to\code{slavedir} and \code{outdir}.

  The name of slave script must be given as argument \code{slave}.  It
  is crucial that this script is present and can be found on each slave
  node. The input file path has to be relative to \code{system.file
  (package = "rpvm")} on each node, and it has to be the same on all
  nodes.  The default input path is the \code{"demo"} subdirectory
  of \code{rpvm} installed directory.

  The output path can be either relative to the home directory or an
  absolute pathname (the default is \code{"/tmp"}).  The output file
  from slave \code{R} process will have the same name as the slave
  script with process id and \code{.Rout} appended to it.

  \code{.PVM.spawnflags} is a mapping of strings to integer constants
  defined by PVM.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.mytid}}
}
\examples{
\dontrun{
children <- .PVM.spawnR (slave = "helloR.R")
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.tasks}
\alias{.PVM.tasks}
\title{Tasks information}
\description{
  Returns information about the tasks running on the virtual machine
}
\usage{
.PVM.tasks (where = 0)
}
\arguments{
  \item{where}{integer specifying what tasks to return information
    about. The options are:
    \itemize{
      \item{0}{for all the tasks on the virtual machine.}
      \item{host id}{for all tasks on a given host.}
      \item{tid}{for a specific task.}
    }
  }
}
\value{
  A data frame.  One row for each task.  Names of the columns are,
  \itemize{
    \item{tid}{task id}
    \item{parent}{parent task id}
    \item{host}{host id}
    \item{status}{task running status (see \code{\link{.PVM.pstats}}).}
    \item{name}{string, name of the spawned task}
  }
}
\details{
 \code{.PVM.tasks} returns information about tasks presently running on
 the virtual machine.  The information returned is the same as that
 available from the pvm console command \code{ps}.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.pstats}}, \code{\link{.PVM.mstats}},
  \code{\link{.PVM.config}}, \code{\link{.PVM.tidtohost}}
}
\examples{
\dontrun{
.PVM.tasks ()
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.tidtohost}
\alias{.PVM.tidtohost}
\title{Host id of a task}
\description{
  Returns the host id which the specified task is running.
}
\usage{
.PVM.tidtohost (tid)
}
\arguments{
  \item{tid}{integer task id specified.}
}
\value{
  Positive integer represents the host id. -1 if failed.
}
\details{
}
\references{
  PVM documentation
}
\examples{
## Find out which host this process in running on
\dontrun{
myhost <- .PVM.tidtohost (.PVM.mytid())
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{.PVM.trecv}
\alias{.PVM.trecv}
\title{Timeout receive}
\description{
  Checks for nonblocking message.
}
\usage{
.PVM.trecv (tid = -1, msgtag = -1, sec = 0, usec = 0)
}
\arguments{
  \item{tid}{tid of sending process (-1 matches any tid)}
  \item{msgtag}{integer message tag (>=0) supplied by the user (-1
    matches any message tag)}
  \item{sec}{time to wait in seconds}
  \item{usec}{time to wait in microseconds}
}
\value{
  Returns the id of the new active receive buffer. -1 indicates an error.
}
\details{
  \code{.PVM.trecv} blocks the process until a message with label
  \code{msgtag} has arrived from \code{tid}.  \code{.PVM.trecv} then
  places the message in a new active receive buffer, also clearing the
  current receive buffer.  If no matching message arrives within the
  specified waiting time, \code{.PVM.trecv} returns without a message.

  \code{sec} and \code{usec} specify how long \code{.PVM.trecv} will
  wait without returning a matching message.  With both set to zero,
  \code{.PVM.trecv} behaves the same as \code{\link{.PVM.nrecv}}, which
  is to probe for messages and return immediately even if none are
  matched.  Setting \code{sec} to -1 makes \code{.PVM.trecv} behave like
  \code{\link{.PVM.trecv}}, that is, it will wait indefinitely.

  If \code{.PVM.trecv} is successful, it will return the new active
  receive buffer identifier.  If no message is received, it returns 0.
  If some error occurs then the return value will be < 0.
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{.PVM.nrecv}}, \code{\link{.PVM.recv}}
}
\examples{
\dontrun{myparent <- .PVM.parent ()}
\dontrun{bufid <- .PVM.trecv (myparent, 4, 2)}
\dontrun{if (bufid > 0) data <- .PVM.upkintvec ()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVM.unpack}
\alias{PVM.unpack}
\alias{.PVM.upkdouble}
\alias{.PVM.upkint}
\alias{.PVM.upkstr}
\alias{.PVM.upkintvec}
\alias{.PVM.upkdblvec}
\alias{.PVM.upkstrvec}
\alias{.PVM.upkintmat}
\alias{.PVM.upkdblmat}
\alias{.PVM.upkstrmat}
\alias{.PVM.upkfactor}
\title{Unpacking data}
\description{
  Unpack data from current active receive buffer.
}
\usage{
.PVM.upkdouble (nitem = 1, stride = 1)
.PVM.upkint (nitem = 1, stride = 1)
.PVM.upkstr (maxlen = 200)

.PVM.upkintvec ()
.PVM.upkdblvec ()
.PVM.upkstrvec ()

.PVM.upkintmat ()
.PVM.upkdblmat ()
.PVM.upkstrmat ()

.PVM.upkfactor ()
}
\arguments{
  \item{nitem}{number of items to unpack.}
  \item{stride}{the stride to be used when unpacking the items.  For
    example, if \code{stride = 2} with \code{.PVM.upkdouble} then every
    other element in the buffer will be unpacked.}
  \item{maxlen}{maximum length of the string buffer}
}
\value{
  Return unpacked data.
}
\details{
  These functions unpack data packed by corresponding \code{.PVM.pk*}
  functions.  The first two functions \code{.PVM.upkdoubl} and
  \code{.PVM.upkint} are lower-level correspondents of the PVM library
  and require the number of items to unpack as an argument.

  \code{.PVM.upkstr} is used to unpack a string send by C functions or
  \code{\link{.PVM.pkstr}}.  It requires an argument \code{maxlen} that
  should be larger than the length of the string passed.  For packing
  and unpacking strings between R processes, \code{\link{.PVM.pkstrvec}}
  and \code{.PVM.upkstrvec} can be used (in the case, the maximum length
  is part of the message).
}
\references{
  PVM documentation
}
\seealso{
  \code{\link{PVM.pack}}
}
\examples{
## Unpack a matrix
\dontrun{a <- .PVM.upkdblmat ()}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{PVMD}
\alias{PVMD}
\alias{.PVM.start.pvmd}
\alias{.PVM.addhosts}
\alias{.PVM.delhosts}
\alias{.PVM.halt}
\title{Vitural Machine Control}
\description{
  Start new pvm daemon, add or delete hosts and stop pvmd.
}
\usage{
.PVM.start.pvmd (hosts = "", block = T)
.PVM.addhosts (hosts)
.PVM.delhosts (hosts)
.PVM.halt ()
}
\arguments{
  \item{hosts}{a character vector, arguments to pvmd}
  \item{block}{logical, if true, block until startup of all hosts complete}
}
\value{
  \code{.PVM.start.pvmd} will return the result from \code{.PVM.config}.
  
  \code{.PVM.halt} returns \code{TRUE} if successful.

  \code{.PVM.addhosts} returns a vector of host ids that has been
  successfully added to the virtual machine.
  
  \code{.PVM.delhosts} returns a vector of status codes for each
  hosts successfulled removed from the virtual machine.
}
\details{
  \code{.PVM.start.pvmd} starts a \emph{pvmd3} process, the master of a
  new virtual machine.  It returns as soon as the pvmd is started and
  ready for work.  If the \code{block} parameter is nonzero and a
  hostfile is passed to the pvmd as a parameter, it returns when all
  hosts marked to start have been added.

  \code{.PVM.addhosts} takes a vector of host names and add them to the
  virtual machine.  The names should have the same syntax as lines of a
  pvmd hostfile (see man page for pvmd3): A hostname followed by options
  of the form xx=y.

  The status of hosts can be requested by the application using
  \code{\link{.PVM.mstats}} and \code{\link{.PVM.config}}.  If a host
  fails it will be automatically deleted from the configuration.  Using
  \code{\link{.PVM.addhosts}} a replacement host can be added by the
  application, however it is the responsibility of the application
  developer to make his application tolerant of host failure.  Another
  use of this feature would be to add more hosts as they become
  available, for example on a weekend, or if the application dynamically
  determines it could use more computational power.

  \code{.PVM.delhosts} deletes the computers pointed to in \code{hosts}
  from the existing configuration of computers making up the virtual
  machine.  All PVM processes and the pvmd running on these computers
  are killed as the computer is deleted.

  If a host fails, the PVM system will continue to function and will
  automatically delete this host from the virtual machine.  An
  application can be notified of a host failure by calling
  \code{\link{.PVM.notify}}.  It is still the responsibility of the
  application developer to make his application tolerant of host
  failure.

  \code{.PVM.halt} shuts down the the entire PVM system including remote
  tasks, remote pvmds, the local tasks (including the calling task) and
  the local pvmd.  Note, when I did this, it also kills the current R
  session.
}
\references{
  PVM documentation
}
\examples{
# start a new virtual machine on local machine 
\dontrun{
.PVM.start.pvmd ()
# add two more hosts to it
.PVM.addhosts ("sparky",
               "thud.cs.utk.edu  ep=$R_LIBS/rpvm/")
# do some work ...
# finished with one machine
.PVM.delhosts ("thud.cs.utk.edu")
# do some other work ...
# finished with pvm
.PVM.halt ()
}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{utilities}
\keyword{connection}
\keyword{interface}

\eof
\name{init.sprng.master}
\alias{init.sprng.master}
\alias{init.sprng.slave}
\alias{init.sprng.group}
\alias{.SPRNG.INIT.TAG}
\title{Interface to rsprng for initializing SPRNG}
\description{
  \code{init.sprng.master} and \code{init.sprng.slave} are used together
  to initialize the parallel random number generator (PRNG) states in
  the RPVM master and slaves processes, respectively.

  \code{init.sprng.group} is used to initialize PRNG states for a RPVM
  group.

  \code{RNGkind ("user")} is called at the end of these functions so
  that the default RNG is substituted with the parallel one.

  \code{.SPRNG.INIT.TAG} is just a prefined message tag number.
}
\usage{
init.sprng.master (children, seed = 0, kindprng = "default",
                   para = 0)
init.sprng.slave  ()
init.sprng.group  (group, rootinst = 0, seed = 0,
                   kindprng = "default", para = 0)
.SPRNG.INIT.TAG <- 199
}
\arguments{
  \item{children}{vector of integer task ids of the slave processes}
  \item{seed}{an integer of random number seed.  It is not the starting
    state of the sequence; rather, it is an encoding of the starting
    state. The same seed for all the streams.  Distinct streams are
    returned.  Only the 31 least significant bits of seed are used in
    determining the initial starting state of the stream} 
  \item{kindprng}{a character string of the disired kind of parallel
    random number generator}
  \item{para}{additional parameters for the parallel random number
    generators.  If \code{para} is 0, default parameters for each PRNG
    are used.  When invalid parameter is given, a warning is issued and
    the default paramter is used.}
  \item{group}{a character string, the name of the group}
  \item{rootinst}{an integer, the root instance number of the group.
    The root instance is responsible for getting the \code{seed},
    \code{rngkind} and \code{para} parameters from the user}
}
\value{
  Return the a two-element character vector of the RNG and normal kinds
  in use before the call.
}
\references{
  SPRNG: Scalable Parallel Random Number Generator Library Web Page.
  \url{http://sprng.cs.fsu.edu/}
}
\seealso{
  \code{\link[rsprng]{init.sprng}}
}
\examples{
# in master script
\dontrun{oldrng <- init.sprng.master (children, seed = 2321)}
# in slave script 
\dontrun{oldrng <- init.sprng.slave ()}
# in group processes
\dontrun{oldrng <- init.sprng.group ("mygroup", root = 0, seed = 1231)}
}
\author{
  Na (Michael) Li \email{nali@umn.edu} and
  A.J. Rossini \email{rossini@u.washington.edu}
}
\keyword{interface}
\keyword{distribution}

\eof
